/* Copyright (c) 2010 Marcin Zajaczkowski
 * All rights reserved.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *      http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package net.sf.appinfo;

import net.sf.appinfo.exception.InitializationException;
import net.sf.appinfo.util.AssertUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * ApplicationInfo implementation which can use the first initialized ApplicationInfo in a chain.
 *
 * Class intended to be directly used in application. It collects one or more ApplicationInfo objects in a chain
 * and returns data from the first initialized (or able to initialize).
 * The first in a chain could be a ApplicationInfo object which reads data from manifest file in JAR/WAR.
 * Later instance using default application info given is static artifical manifest file placed in resources
 * for use in development versions run from IDE. To be at the end of a chain FallbackApplicationInfo class was designed.
 * Its initialization cannot fail nad therefor there is no risk to prevent IoC context from start.
 *
 * There is a default appInfo's Spring bean configuration in resources (appInfoDefaultContext.xml).
 * It can be add to application's Spring conext configuration. More information is placed inside xml.
 *
 * @see net.sf.appinfo.AbstractInitAwareApplicationInfo
 * @see net.sf.appinfo.InitAwareApplicationInfo
 * @see net.sf.appinfo.FallbackApplicationInfo
 * 
 * @since 0.4.0
 *
 * @author Marcin Zajączkowski, 2010-10-08
 */
public class ChainedApplicationInfo extends AbstractInitAwareApplicationInfo {

    protected final Logger logger = LoggerFactory.getLogger(getClass());

    private List<InitAwareApplicationInfo> appInfos = new ArrayList<InitAwareApplicationInfo>();

    public ChainedApplicationInfo(InitAwareApplicationInfo... appInfos) {
        super();
        AssertUtil.notNull(appInfos, "appInfos");
        Collections.addAll(this.appInfos, appInfos);
    }

    /** {@inheritDoc} */
    @Override
    protected void initClass() throws InitializationException {
        boolean anyInited = false;
        for (InitAwareApplicationInfo appInfo : appInfos) {
            //TODO: MZA: Think about refactoring
            try {
                appInfo.initInfo();
                setValuesFromGivenAppInfo(appInfo);
                anyInited = true;
                break;
            } catch (InitializationException e) {
                logger.warn("Unable to initialize appInfo ({}) in chain. Error message: {}",
                        appInfo.getClass().getSimpleName(), e.getMessage());
            }
        }
        if (!anyInited) {
            throw new InitializationException(
                    "Unable to initialize any of " + appInfos.size() + " AppInfo available in chain");
        }
    }

    //Could be moved to Utils class and make static
    private void setValuesFromGivenAppInfo(InitAwareApplicationInfo appInfo) {
        setName(appInfo.getName());
        setVersion(appInfo.getVersion());
        setBuildNumber(appInfo.getBuildNumber());
        setBuildDate(appInfo.getBuildDate());
        setScmRevision(appInfo.getScmRevision());
    }
}
